/*******************************************************************************
*
*	Program Name:	LecTesterNET.exe
*	Module Name:	LecTesterSession.cs
*
*	Cambridge Technology Inc. makes no warranties, either explicit or implied, 
*	with regards to the applicability of these sample source codes for an end-user 
*	application.  They are provided merely as examples on how to interface with the 
*	EC1000 API.
*
*	Description:
*		Program to test the NET interface to the EC1000 Clients
*		LecBroadcast.dll and LecSession.dll
*
*   Author:   JLV
* 
*   Environment: .NET
* 
*   Compiler:
*       - tested with Visual Studio .NET 2003
*
*******************************************************************************/

using System;
using System.Threading;
using System.IO;
using System.Windows.Forms;
using Microsoft.Win32;

using LanmarkControls.LECBroadcast;
using LanmarkControls.LECSession;


/*****************************************************************************/
public class ECSession
{

#if DEBUG
	uint	LEC_TIMEOUT  = 1500;
#else
	uint 	LEC_TIMEOUT = 15;
#endif

	Session cS = null;	// create a Session Obj Instance

	public const int FIXDATACOUNT = 8;

	public enum eFixDataType
	{
		ADMIN=0x0000000A, 
		CONTROL=0x00000005, 
		LENS=0x00000002, 
		LASER=0x00000006, 
		USER=0x0000000F, 
		CORRECTION=0x0000000D,
		CORRECTION2=0x1000000D,
		PERFORMANCE=0x00000010,
	}

	private int[]	iFixDataType = new int[FIXDATACOUNT]
	{   
		(int)eFixDataType.ADMIN, 
		(int)eFixDataType.CONTROL, 
		(int)eFixDataType.LENS, 
		(int)eFixDataType.LASER, 
		(int)eFixDataType.USER,  
		(int)eFixDataType.CORRECTION,
		(int)eFixDataType.CORRECTION2,
		(int)eFixDataType.PERFORMANCE 
	};
	public  ECSession.eFixDataType FixDataType(int i) {  return (ECSession.eFixDataType)iFixDataType[i];  }
	
	private string[] strStorageName = new string[FIXDATACOUNT]
	{ 
		"AdminConfig",           
		"LECControlConfig",             
		"LECLens",            
		"laser00", 
		"LecUser", 
		"NoCorrection", 
		"NoCorrection", 
		"GlobalConfig" 
	};
	public   string StorageName(int i) {  return strStorageName[i];  }


    bool bDataEventArrived = false;
    bool bFixDataEventArrived = false;

	public string m_LocalIPAddress
	{
		get { return _LocalIPAddress; }
		set{ _LocalIPAddress = value; }
	}
	private string	_LocalIPAddress;

	public string Address { get { return _strAddress; } }
	private string	_strAddress;

	public int PortNumber { get { return _piPortNumber; } }
	private int		_piPortNumber;


	/*--------------------------------------------*/
	// Contractor initializes values of class instance
	public ECSession(string strLocalIPAddress, string strAddress, int piPortNumber)
	{
		_strAddress = strAddress;
		_piPortNumber = piPortNumber;
		_LocalIPAddress = strLocalIPAddress;

		Console.WriteLine("\n\nCreating SESSION object. ");
		cS = new Session();	// create a Session Obj Instance

        // Attach to receive Events from Session server by using "+=" 
        //
        cS.OnMessageEvent += new MessageEventHandler(cS_OnMessageEvent);
        cS.OnDataEvent += new DataEventHandler(cS_OnDataEvent);
        cS.OnConnectionEvent += new ConnectionEventHandler(cS_OnConnectionEvent);

    }

	/*--------------------------------------------*/
	public bool Attach()
	{
        string strUserName = "";
        string strPassword = "";

        if (false)
        {
            Console.Write("\n\nEnter USERNAME > ");
            strUserName = Console.ReadLine();
            Console.Write("\n\nEnter PASSWORD > ");
            strPassword = Console.ReadLine();
            Console.WriteLine("\nAbout to loginSession( Username='{0}'  Password='{1}') .......", strUserName, strPassword);
        }

        Console.WriteLine("\nAbout to loginSession( ) .......");

		// Login to Start Session //////////////////////////////////////////
		//
        uint err = cS.loginSession(_LocalIPAddress, _strAddress, _piPortNumber, strUserName, strPassword, LEC_TIMEOUT);

		ECUtils.DisplayErr(err, "cS.loginSession", _strAddress, Convert.ToString(_piPortNumber), "");
		if (0 != err)
		{
			MessageBox.Show("ERROR: failed to login to target: " + _strAddress, "LecSession client", MessageBoxButtons.OK);
			return false;
		}
		Thread.Sleep(1000);

		return true;
	}

	/*--------------------------------------------*/
	public bool Detach() 
	{ 
		// Logout to End Session //////////////////////////////////////////
		//
		Console.WriteLine("\nAbout to logoutSession( ) .......");

		uint err = cS.logoutSession(LEC_TIMEOUT);

        ECUtils.DisplayErr(err, "cS.logoutSession", _strAddress, "", "");

		return true;
	}

	/*--------------------------------------------*/

	public bool SessionTest() 
	{ 

		DialogResult result;
		string	pstrData = "";
		string	sTemp = "5";
		uint	err = 0;
		int		iAlternate = 0;

		try
		{
			do 
			{
                // Get all available FixData  //////////////////////////////////////////
                //
                if (DialogResult.Yes ==
                    MessageBox.Show("Do you want to Read & Write back all FixData ?", "LecTester Program", MessageBoxButtons.YesNo))
                {
                    for (int i = 0; i < FIXDATACOUNT; i++)	// get Session Fix Data
                    {
                        err = ReadFixData(out pstrData, i);

                        if (0 == err)
                        {
                            // Send back FixData  //////////////////////////////////////////
                            WriteFixData(pstrData, i);
                        }
                    }
                }
				// Send Correction via Stream Data  //////////////////////////////////////////
				//
				result = MessageBox.Show("Do you want to Send (45 Rotation) CorrectionData via sendStreamData() ?", "LecTester Program", MessageBoxButtons.YesNo);
				if (result == System.Windows.Forms.DialogResult.Yes)
				{
                    try
                    {
                        System.IO.StreamReader file = new System.IO.StreamReader(@".\D2_012_rotated.xml");
                        pstrData = file.ReadToEnd();

                        err = cS.sendStreamData(pstrData, LEC_TIMEOUT);
                        ECUtils.DisplayErr(err, "cS.sendStreamData", "CorrectionData", "", pstrData);
                    }
                    catch (Exception e)
                    {
                        ECUtils.LogWrite(e.Message);
                    }
                }

				// Send Job Stream Data  //////////////////////////////////////////
				//
				int iLoopCnt = 2;
				Console.Write("\n\nEnter number of times to sendStreamData( ) test (1) > ");
				sTemp = Console.ReadLine();
				if ("" == sTemp)	iLoopCnt = 1;
				else				iLoopCnt = Convert.ToInt32(sTemp);
				Console.Write("\n\nCalling sendStreamData( ) {0} times. \n", iLoopCnt);

				string strTstJobStream = ECUtils.strTstJobStream;
				//if (iAlternate++ % 2 == 0)
				//	strTstJobStream = ECUtils.strTstJobStream1;
                try
                {
                    StreamReader NewReader = new StreamReader(@".\TestJobData.xml");
                    strTstJobStream = NewReader.ReadToEnd();
                    NewReader.Close();
                }
                catch (Exception e)
                {
                    ECUtils.LogWrite(e.Message);
                }

				for (int i=0; i<iLoopCnt; i++)	// 
				{
					err = cS.sendStreamData(strTstJobStream, LEC_TIMEOUT);
                    ECUtils.DisplayErr(err, "cS.sendStreamData", Convert.ToString(i), "", strTstJobStream);
					//					Thread.Sleep(100);
				}

                // Save Job  Data  //////////////////////////////////////////
                //
                err = cS.saveJobData(1, "", strTstJobStream, 1, LEC_TIMEOUT);
                ECUtils.DisplayErr(err, "cS.saveJobData", "", "", strTstJobStream);

                int iJobCnt = 0;
                string strJobList;
                err = cS.requestJobNameList(1, out iJobCnt, out strJobList, LEC_TIMEOUT);
                ECUtils.DisplayErr(err, "cS.requestJobNameList", iJobCnt.ToString(), "", strJobList);

                err = cS.requestJobNameList(2, out iJobCnt, out strJobList, LEC_TIMEOUT);
                ECUtils.DisplayErr(err, "cS.requestJobNameList", iJobCnt.ToString(), "", strJobList);


				// Send Priority Data  //////////////////////////////////////////
				//
				string	strPriority;
				result = MessageBox.Show("Do you want to SUSPEND JOB ?", "LecTester Program", MessageBoxButtons.YesNo);
				if (result == System.Windows.Forms.DialogResult.Yes)
				{
					strPriority = @"<Data type='ServiceData'> <Msg id='Suspend'/> </Data>";
					err = cS.sendPriorityData(strPriority, LEC_TIMEOUT);
                    ECUtils.DisplayErr(err, "cS.sendPriorityData", "SUSPEND JOB", "", "");
				}

				result = MessageBox.Show("Do you want to RESUME JOB ?", "LecTester Program", MessageBoxButtons.YesNo);
				if (result == System.Windows.Forms.DialogResult.Yes)
				{
					strPriority = @"<Data type='ServiceData'> <Msg id='Resume'/> </Data>";
					err = cS.sendPriorityData(strPriority, LEC_TIMEOUT);
                    ECUtils.DisplayErr(err, "cS.sendPriorityData", "RESUME JOB", "", "");
				}

                result = MessageBox.Show("Do you want to ABORT JOB ?", "LecTester Program", MessageBoxButtons.YesNo);
                if (result == System.Windows.Forms.DialogResult.Yes)
                {
                    strPriority = @"<Data type='ServiceData' rev='1.0'> <Msg id='Abort' reason='Job'/> </Data>";
                    err = cS.sendPriorityData(strPriority, LEC_TIMEOUT);
                    ECUtils.DisplayErr(err, "cS.sendPriorityData", "ABORT JOB", "", "");
                    Thread.Sleep(1000);
                }

				result = MessageBox.Show("Do you want to SET INTERLOCKS ?", "LecTester Program", MessageBoxButtons.YesNo);
				if (result == System.Windows.Forms.DialogResult.Yes)
				{
                    //strPriority = @"<Data type='ServiceData' rev='1.0'> <Msg id='SetInterlockEnable' config='0x01'/> </Data>";
                    strPriority = @"<Data type='ServiceData' rev='1.0'> <Msg id='SetInterlockEnable' config='0x11'/> </Data>";
                    err = cS.sendPriorityData(strPriority, LEC_TIMEOUT);
                    ECUtils.DisplayErr(err, "cS.sendPriorityData", "SetInterlockEnable", "", "");
				}

				result = MessageBox.Show("Do you want to GET FPGA REGISTERS ?", "LecTester Program", MessageBoxButtons.YesNo);
				if (result == System.Windows.Forms.DialogResult.Yes)
				{
                    strPriority = @"<Data type='ServiceData' rev='1.0'> <Msg id='GetRegisters'/> </Data>";
                    /* Read the initial time. */
					DateTime startTime = DateTime.Now;
					Console.WriteLine("repeat GetRegisters 10 times\n"+ startTime);

					// loop 10 times to read Register data
					for (int i=0; i<10; i++)
					{
						bDataEventArrived = false;
						err = cS.sendPriorityData(strPriority, LEC_TIMEOUT);
                        ECUtils.DisplayErr(err, "cS.sendPriorityData", "GetRegisters", "", "");

						DateTime startTime1 = DateTime.Now;
						int cnt=0;
						while(!bDataEventArrived)
						{
							Thread.Sleep(10);
							if (cnt++ > 200)
							{
								Console.WriteLine("TIMEOUT: waiting for DataEvent with Registers data.");
								break;
							}
						}
						DateTime stopTime1 = DateTime.Now;
						TimeSpan duration1 = stopTime1 - startTime1;
						Console.Write(duration1 + "  ");
					}
					/* Read the end time. */
					DateTime stopTime = DateTime.Now;
					Console.WriteLine(stopTime);

					/* Compute the duration between the initial and the end time. */
					TimeSpan duration = stopTime - startTime;
					Console.WriteLine(duration);
				}

			} while( DialogResult.Yes ==
				MessageBox.Show("**** DO YOU WANT TO REPEAT TESTS ?  ******", "LecTester Program", MessageBoxButtons.YesNo) );

            result = MessageBox.Show("Do you want to REBOOT/RESTART target ?", "LecTester Program", MessageBoxButtons.YesNo);
            if (result == System.Windows.Forms.DialogResult.Yes)
            {
                string strPriority = @"<Data type='ServiceData'> <Msg id='Restart'/> </Data>";
                err = cS.sendPriorityData(strPriority, LEC_TIMEOUT);
                ECUtils.DisplayErr(err, "cS.sendPriorityData", "RESTART TARGET", "", "");
                Thread.Sleep(1000);
            }

        }
		catch(Exception ex)
		{
			Console.WriteLine("Unexpected LECSession Server exception: " + ex.Message);
			Console.WriteLine("\n\n Exiting test.  logoutSession( ) ......."); 
			cS.logoutSession(LEC_TIMEOUT);
		}

		return true;
	}


	/*--------------------------------------------*/

	// get new StorageNames from LecControlConfig.xml file
	public void UpdateStorageNames(string strFixData)
	{
		ECUtils.ReadFromXML(strFixData, 0, "LensFile",  null, out strStorageName[2]);
		ECUtils.ReadFromXML(strFixData, 0, "LaserFile", null, out strStorageName[3]);
		ECUtils.ReadFromXML(strFixData, 0, "UserFile",  null, out strStorageName[4]);
		ECUtils.ReadFromXML(strFixData, 0, "CorrFile1",  null, out strStorageName[5]);
		ECUtils.ReadFromXML(strFixData, 0, "CorrFile2",  null, out strStorageName[6]);
		ECUtils.ReadFromXML(strFixData, 0, "PerformanceFile",  null, out strStorageName[7]);
		Console.WriteLine("\nUpdateStorageNames: ");
		for (int ii=0; ii<FIXDATACOUNT; ii++)
			Console.WriteLine(strStorageName[ii]+ " ");
	}

	/*--------------------------------------------*/

	public uint ReadFixData(out string strFixData , int i)
	{
		strFixData = "";
		uint	err = 1;
		int		j = 0;

        if (i >= FIXDATACOUNT)  return err;

		while (0 != err)
		{
			err = cS.requestFixedData(iFixDataType[i] & 0xFFFF, strStorageName[i], out strFixData, LEC_TIMEOUT);

            ECUtils.DisplayErr(err, "cS.requestFixedData", strStorageName[i], iFixDataType[i].ToString("X").PadLeft(8, '0'), strFixData);
            if (0 == err)
			{
                // get new StorageNames from LecControlConfig.xml file
                if (iFixDataType[i] == (int)eFixDataType.CONTROL)
					UpdateStorageNames(strFixData);
			}
			else
			{
				Thread.Sleep(1000);
				if (++j>1)	break;
			}
		}

		return err;
	}

	/*--------------------------------------------*/

	public uint WriteFixData(string strFixData , int i)
	{
		uint	err = 1;
		int		j = 0;
        int[]   iTimeout = new int[FIXDATACOUNT] {10,10,10,10,10,200,200,10};

		while (0 != err)
		{
            bFixDataEventArrived = false;
            err = cS.sendFixedData(strFixData, strStorageName[i], LEC_TIMEOUT);

            ECUtils.DisplayErr(err, "cS.sendFixedData", strStorageName[i], iFixDataType[i].ToString("X").PadLeft(8, '0'), strFixData);

            int cnt = 0;
            while (!bFixDataEventArrived)
            {
                Thread.Sleep(100);
                if (cnt++ > iTimeout[i])
                {
                    Console.WriteLine("TIMEOUT: {0} sec waiting for FixData Event Done.", iTimeout[i]/10);
                    break;
                }
            }

            if (0 != err)
			{
				Thread.Sleep(1000);
				if (++j>2)	break;
			}
		}

		return err;
	}

	/*--------------------------------------------*/

	private void cS_OnConnectionEvent(string pstrName, bool pbConnected)
	{
		ECUtils.ConsoleChangeColor();
		Console.WriteLine("<<=== cS_OnConnectionEvent:  pstrName= {0} pbConnected= {1}\n",pstrName, pbConnected);
		ECUtils.ConsoleRestoreColor();
	}

	/*--------------------------------------------*/

	private void cS_OnMessageEvent(uint puiPayloadHigh, uint puiPayloadLow)
	{
#if _EVENT_DEBUG
		uint uiPayLoadHighHI = puiPayloadHigh >> 16;

		ECUtils.ConsoleChangeColor();


		switch (puiPayloadHigh & 0xFFFF)
		{
			case 0x41:		// Busy w/ Job
				Console.Write(" <=Busy");
				break;
		
			case 0x42:		// done w/ Job
				Console.Write(" <=Free");
				break;
		
			case 0x5040:		// 20544 Application Event
			switch (uiPayLoadHighHI)
			{
				case 0x06:		// Busy w/ Job
				case 0x41:		// Busy w/ Job
					Console.Write(" <=Busy{0}", uiPayLoadHighHI);
					break;
		
				case 0x07:	 	// not Busy w/ Job
				case 0x42:		// not Busy w/ Job
					Console.Write(" <=Free 0x{0:X4}", uiPayLoadHighHI);
					break;

                case 0x0F:		// 15 FixDataDone
                    Console.Write(" <=FixDataDone");
                    bFixDataEventArrived = true;
                    break;
		
				case 0x2329:		// 9001 Aborted Job
					Console.Write(" <=Aborted");
                    break;
		
				case 0x233D:		// 9021 Interlock Job
					Console.Write(" <=Interlock 0x{0:X4}", puiPayloadLow & 0xFFFF);
					break;
		
				case 0xA678:		// 42616 Job Exceptions
				switch (puiPayloadLow & 0xFFFF)
				{
					case 0x2333:		// WaitForIOTimeout
						Console.Write(" <=Wait4IO_Timeout");
						break;
					default:
						Console.Write(" <=Exeption 0x{0:X4}", puiPayloadLow & 0xFFFF);
						break;
				}// end switch
					break;
				default:
					Console.WriteLine("\n  <== Exception: PayLoadHigh= Lo={0}(0x{1:X4}) Hi={2}(0x{3:X4}),\n\tPayLoadLow= {4}(0x{5:X8}) ", 
						puiPayloadHigh & 0xFFFF,  puiPayloadHigh & 0xFFFF,  
						puiPayloadHigh >> 16,  puiPayloadHigh >> 16,  
						puiPayloadLow,  puiPayloadLow);
					break;
			}// end switch
				break;
		}// end switch

#if DEBUG
		Console.WriteLine("\n  <== OnMessageEvent: PayLoadHigh= Lo={0}(0x{1:X4}) Hi={2}(0x{3:X4}),\n\tPayLoadLow= {4}(0x{5:X8}) ", 
			puiPayloadHigh & 0xFFFF,  puiPayloadHigh & 0xFFFF,  
			puiPayloadHigh >> 16,  puiPayloadHigh >> 16,  
			puiPayloadLow,  puiPayloadLow);
#endif

		ECUtils.ConsoleRestoreColor();
#endif
		return;
	}


	/*--------------------------------------------*/
	private void cS_OnDataEvent(uint puiDataID, uint puiErrorCode, string pstrData)
	{
/**/
		ECUtils.ConsoleChangeColor();
#if _EVENT_DEBUG
		Console.WriteLine("<<=== cS_OnDataEvent:  puiDataID= {0} puiErrorCode= {1} \n{2}\n",
			Convert.ToString(puiDataID,16), Convert.ToString(puiErrorCode,16), pstrData);
#else
		Console.Write(" <= {0} ", Convert.ToString(puiDataID>>16, 16) );
#endif
		ECUtils.ConsoleRestoreColor();
/**/
		bDataEventArrived = true;
	}

	/*--------------------------------------------*/

}


