/*******************************************************************************
*
*	Program Name:	LecTesterCOM.exe and others
*	Module Name:	LecTesterUtils.cs
*
*	Cambridge Technology Inc. makes no warranties, either explicit or implied, 
*	with regards to the applicability of these sample source codes for an end-user 
*	application.  They are provided merely as examples on how to interface with the 
*	EC1000 API.
*
*	Description:
*		Utilities routines to access XML data, Local IP address, etc
*
*   Author:   JLV
* 
*   Environment: Win32
* 
*   Compiler:
*       - tested with Visual C++ 6.0 and Visual Studio .NET 2003
*
*******************************************************************************/

using System;
using System.IO;
using System.Xml;
using System.Xml.Schema;
using System.Windows.Forms;
using Microsoft.Win32;
using System.Runtime.InteropServices; // DllImport()




/*****************************************************************************/
public class ECUtils
{
	[DllImport("kernel32.dll", SetLastError=true)]
	public static extern bool SetConsoleTextAttribute(
		IntPtr hConsoleOutput,
		CharacterAttributes wAttributes);
	/* declaring the setconsoletextattribute function*/

	[DllImport("kernel32.dll")]
	public static extern IntPtr GetStdHandle(int nStdHandle);
	/*declaring the getstdhandle funtion to get thehandle 
		  that would be used in setConsoletextattribute function */

#if DEBUG
	[DllImport ( "kernel32.dll", SetLastError = true )]
	[return: MarshalAs ( UnmanagedType.Bool )]
	static extern bool AllocConsole ( );

	[DllImport ( "kernel32.dll", SetLastError = true )]
	[return: MarshalAs ( UnmanagedType.Bool )]
	static extern bool FreeConsole ( );
#endif

	/////////////////////////////////////////////////////////////////////////

	public static int  ConsoleInit()
	{
        if ((IntPtr)null == hConsole)
        {
#if DEBUG
            AllocConsole();
#endif
            int STD_OUTPUT_HANDLE = -11;

            hConsole = GetStdHandle(STD_OUTPUT_HANDLE);
#if DEBUG
            Console.Write("\nConsoleInit() handle = " + hConsole.ToString() + " \n");
#endif
        }

		return 0;
	}

    /////////////////////////////////////////////////////////////////////////

    public static int ConsoleEnd()
    {
        if ((IntPtr)null != hConsole)
        {
            SetConsoleTextAttribute(hConsole, CharacterAttributes.FOREGROUND_WHITE);

#if DEBUG
            if (FreeConsole()) return 1;
#endif
            hConsole = (IntPtr)null;
        }
        return 0;
    }

    /////////////////////////////////////////////////////////////////////////

	public static bool  ConsoleChangeColor()
	{
		bool ret = true;

        if ((IntPtr)null != hConsole)
        {
            ret = SetConsoleTextAttribute(hConsole,
                CharacterAttributes.FOREGROUND_RED | CharacterAttributes.FOREGROUND_INTENSITY);
        }

        return ret;
	}

	/////////////////////////////////////////////////////////////////////////

	public static bool   ConsoleRestoreColor()
	{
		bool ret = true;

        if ((IntPtr)null != hConsole)
        {
            ret = SetConsoleTextAttribute(hConsole, CharacterAttributes.FOREGROUND_WHITE);
        }

        return ret;
	}

    /////////////////////////////////////////////////////////////////////////
    public static void LogOpen(string strName)
    {
        if (m_oLog == null)
        {
            m_oLog = new StreamWriter(strName, false);
            m_oLog.AutoFlush = true;
        }

        m_oLog.WriteLine("EC1000 Firmware Loader log file.");
    }

    /////////////////////////////////////////////////////////////////////////
    public static void LogClose()
    {
        if (m_oLog != null)
        {
            m_oLog.Flush();
            m_oLog.Close();
            m_oLog = null;
        }
    }

    /////////////////////////////////////////////////////////////////////////
    public static void LogWrite(string str)
    {
        Console.WriteLine(str);

        if (m_oLog != null)
        {
            m_oLog.WriteLine(str);
        }
    }

    /////////////////////////////////////////////////////////////////////////

/*


		public const uint Success = 0;
		public const uint Error_AccessDenied = 1;
		public const uint Error_CommunicationLink = 2;
		public const uint Error_NotConnected = 3;

		public const uint Error_IllegalClientId = 4;
		public const uint Error_InvalidPersistState = 5;
		public const uint Error_ServerNameNotFound = 8;

		public const uint Error_InvalidParameter = 9;
		public const uint Error_Network = 10;
		public const uint Error_DataNotFound = 11;
		public const uint Error_PathNotFound = 12;
		public const uint Error_Access = 13;
		public const uint Error_LocalAccess = 14;
		public const uint Error_DataUnknown = 15;
		public const uint Error_EventHandling = 16;
		public const uint Error_NotAvailable = 17;
		public const uint Error_CancelByUser = 18;
		public const uint Error_Aborting = 19;
		public const uint Error_Aborted = 20;
		public const uint Error_LicenseUnavailable = 21;
		public const uint Error_LicenseAccessDenied = 22;
		public const uint Error_Exception = 23;
		public const uint Error_Timeout = 24;
		public const uint Error_NoData = 25;
		public const uint Error_DataExists = 26;
		public const uint Error_LaserFile = 27;
		public const uint Error_RemoteAccess = 28;
		public const uint Error_StateError = 29;
		public const uint Error_NotFound = 30;
		public const uint Buffer_Full = 31;
*/

    public static void DisplayErr(uint err, string strFnctionName, string strParam1, string strParam2, string strRawData)
	{
		if (0 == err)
		{
            ECUtils.LogWrite("\n SUCCESS " + strFnctionName.ToString() + "( " + strParam1.ToString() + ", " + strParam2.ToString() + " )  SUCCESS\n");
		}
		else
		{
            ECUtils.LogWrite("FAILED  " + strFnctionName.ToString() + "( " + strParam1.ToString() + ", " + strParam2.ToString() + " ) FAILED err= "+err.ToString()+"\n");
			switch(err)
			{
				case 1 :
					ECUtils.LogWrite("ERROR: "+err.ToString()+" - AccessDenied  \n");
					break;
				case 2 :
					ECUtils.LogWrite("ERROR: "+err.ToString()+" - CommunicationLink.  Could not establish a connection  \n");
					break;
				case 3 :
					ECUtils.LogWrite("ERROR: "+err.ToString()+" - NotConnected. Licensing is dening access to Broadcast Feature  \n");
					break;
				case 4 :
					ECUtils.LogWrite("ERROR: "+err.ToString()+" - Illegal client identifier  \n");
					break;
				case 5 :
					ECUtils.LogWrite("ERROR: "+err.ToString()+" - InvalidPersistState \n");
					break;
				case 8 :
					ECUtils.LogWrite("ERROR: "+err.ToString()+" - Server name not found  \n");
					break;
				case 9 :
					ECUtils.LogWrite("ERROR: "+err.ToString()+" - InvalidParameter  \n");
					break;
				case 10 :
					ECUtils.LogWrite("ERROR: "+err.ToString()+" - Network connection not established  \n");
					break;
				case 11 :
					ECUtils.LogWrite("ERROR: "+err.ToString()+" - Requested data not found in EC device  \n");
					break;
				case 12 :
					ECUtils.LogWrite("ERROR: "+err.ToString()+" - Location of data not found in EC device  \n");
					break;
				case 13 :
					ECUtils.LogWrite("ERROR: "+err.ToString()+" - Canot access remote location of where the data resides/written on the EC device  \n");
					break;
				case 14 :
					ECUtils.LogWrite("ERROR: "+err.ToString()+" - LocalAccess. Canot access local destination/source location where the data is resides/written to  \n");
					break;
				case 15 :
					ECUtils.LogWrite("ERROR: "+err.ToString()+" - The data type requested is unknown or error in data format \n");
					break;
				case 16 :
					ECUtils.LogWrite("ERROR: "+err.ToString()+" - Could not establish Event handling  \n");
					break;
				case 17 :
					ECUtils.LogWrite("ERROR: "+err.ToString()+" -  NotAvailable \n");
					break;
				case 18 :
					ECUtils.LogWrite("ERROR: "+err.ToString()+" - Cancelled by user  \n");
					break;
				case 19 :
					ECUtils.LogWrite("ERROR: "+err.ToString()+" -  Aborting \n");
					break;
				case 20 :
					ECUtils.LogWrite("ERROR: "+err.ToString()+" -  Aborted \n");
					break;
				case 21 :
					ECUtils.LogWrite("ERROR: "+err.ToString()+" -  LicenseUnavailable \n");
					break;
				case 22 :
					ECUtils.LogWrite("ERROR: "+err.ToString()+" -  LicenseAccessDenied \n");
					break;
				case 23 :
					ECUtils.LogWrite("ERROR: "+err.ToString()+" -  Exception. s/w error \n");
					break;
				case 24 :
					ECUtils.LogWrite("ERROR: "+err.ToString()+" -  Timeout \n");
					break;
				case 25 :
					ECUtils.LogWrite("ERROR: "+err.ToString()+" -  NoData \n");
					break;
				case 26 :
					ECUtils.LogWrite("ERROR: "+err.ToString()+" -  DataExists \n");
					break;
				case 27 :
					ECUtils.LogWrite("ERROR: "+err.ToString()+" -  LaserFile \n");
					break;
				case 28 :
					ECUtils.LogWrite("ERROR: "+err.ToString()+" -  RemoteAccess \n");
					break;
				case 29 :
					ECUtils.LogWrite("ERROR: "+err.ToString()+" -  StateError \n");
					break;
				case 30 :
					ECUtils.LogWrite("ERROR: "+err.ToString()+" -  NotFound \n");
					break;
				case 31 :
					ECUtils.LogWrite("ERROR: "+err.ToString()+" -  Buffer full \n");
					break;
				default:
					ECUtils.LogWrite("ERROR: "+err.ToString()+" - Unknown error\n");
					break;
			}
		}
#if DEBUG
        ECUtils.LogWrite(strRawData);
#endif
	}


	/////////////////////////////////////////////////////////////////////////

	public static void GetLocalIPAddress(out string strLocalIPAddress)
	{
		RegistryKey OurKey = Registry.LocalMachine;
		OurKey = OurKey.OpenSubKey("SYSTEM\\CurrentControlSet\\Services\\Tcpip\\Parameters\\Interfaces", true);

		strLocalIPAddress = "0.0.0.0";
		int	iCnt = 0;
        bool bValidDHCP = false;

		//The first example shows it using a foreach loop to display each subkeyname
		foreach(string Keyname in OurKey.GetSubKeyNames())
		{
            bValidDHCP = false;
            
            RegistryKey OurSubKey = OurKey.OpenSubKey(Keyname, true);

			if (null != OurSubKey.GetValue("DhcpIPAddress") &&
                   0 != OurSubKey.GetValue("DhcpIPAddress").ToString().CompareTo("0.0.0.0") )
			{
				if (null != OurSubKey.GetValue("EnableDHCP")  &&
                     "1" == OurSubKey.GetValue("EnableDHCP").ToString() )
				{
                    if (null != OurSubKey.GetValue("DhcpRetryStatus"))
                    {
                        bValidDHCP = true;

                        ECUtils.LogWrite("Found one of your Local IP addresses: " + OurSubKey.GetValue("DhcpIPAddress").ToString() + " ...\n");
                        iCnt++;
                        if (iCnt == 1)	// if only one found, no need to ask user to choose
                        {
                            strLocalIPAddress = OurSubKey.GetValue("DhcpIPAddress").ToString();
                        }
                        if (iCnt == 2)
                        {
                            string message = "Found multiple IP addresses in your PC\n Is Local IPAddress=  "
                                + strLocalIPAddress + "   Connected to EC1000 ? ";
                            DialogResult result = MessageBox.Show(message, "LecTester program", MessageBoxButtons.YesNo);
                            if (result == System.Windows.Forms.DialogResult.Yes)
                                break;
                        }
                        if (iCnt > 1)
                        {
                            strLocalIPAddress = "0.0.0.0";
                            string message = "Found multiple IP addresses in your PC\n Is Local IPAddress=  "
                                + OurSubKey.GetValue("DhcpIPAddress").ToString() + "   Connected to EC1000 ? ";
                            DialogResult result = MessageBox.Show(message, "LecTester program", MessageBoxButtons.YesNo);
                            if (result == System.Windows.Forms.DialogResult.Yes)
                            {
                                strLocalIPAddress = OurSubKey.GetValue("DhcpIPAddress").ToString();
                                break;
                            }
                        }
                    }
                }
            }

            if (! bValidDHCP  &&
                null != OurSubKey.GetValue("IPAddress") && 
                   0 != ((string[])OurSubKey.GetValue("IPAddress"))[0].CompareTo("0.0.0.0") )
            {
                string strIPAddress = ((string[])OurSubKey.GetValue("IPAddress"))[0];
                //if (null != OurSubKey.GetValue("DhcpRetryStatus"))
                {
                    ECUtils.LogWrite("Found one of your Local IP addresses: " + strIPAddress + " ...\n");
                    iCnt++;
                    if (iCnt == 1)	// if only one found, no need to ask user to choose
                    {
                        strLocalIPAddress = strIPAddress;
                    }
                    if (iCnt == 2)
                    {
                        string message = "Found multiple IP addresses in your PC\n Is Local IPAddress=  "
                            + strLocalIPAddress + "   Connected to EC1000 ? ";
                        DialogResult result = MessageBox.Show(message, "LecTester program", MessageBoxButtons.YesNo);
                        if (result == System.Windows.Forms.DialogResult.Yes)
                            break;
                    }
                    if (iCnt > 1)
                    {
                        strLocalIPAddress = "0.0.0.0";
                        string message = "Found multiple IP addresses in your PC\n Is Local IPAddress=  "
                            + strIPAddress + "   Connected to EC1000 ? ";
                        DialogResult result = MessageBox.Show(message, "LecTester program", MessageBoxButtons.YesNo);
                        if (result == System.Windows.Forms.DialogResult.Yes)
                        {
                            strLocalIPAddress = strIPAddress;
                            break;
                        }
                    }
                }
            }
            OurSubKey.Close();
		} 
		OurKey.Close();

		return;
	}

	/////////////////////////////////////////////////////////////////////////

	public static void ReadFromXML(StreamReader poData,  int iElementSkipCount,
		string strSearchElement, string strSearchAttribute, 
		out string strOutString)
	{
		strOutString = "";

		XmlTextReader xmlRead = new XmlTextReader(poData);

		ReadFromXML(xmlRead, iElementSkipCount, strSearchElement, strSearchAttribute, out strOutString);

		xmlRead.Close();
	}

	/////////////////////////////////////////////////////////////////////////
	public static void ReadFromXML(string pstrData,  int iElementSkipCount,
		string strSearchElement, string strSearchAttribute, 
		out string strOutString)
	{
		strOutString = "";

		if (pstrData.Length == 0)
			return;

		StringReader strRead = new StringReader(pstrData);
		XmlTextReader xmlRead = new XmlTextReader(strRead);

		ReadFromXML(xmlRead, iElementSkipCount, strSearchElement, strSearchAttribute, out strOutString);

		xmlRead.Close();
		strRead.Close();
	}

	/////////////////////////////////////////////////////////////////////////
	// Reads either an Element Text Value or an Element's Attribute.
	// If Attribute search param is set to null then we are looking 
	// for Element Text Value
	//
	public static void ReadFromXML(XmlTextReader pxmlRead, int iElementSkipCount,
		string strSearchElement, string strSearchAttribute, 
		out string strOutString)
	{
		bool fContinueSearch = true;
		bool fElementFound = false;
		int	iCnt = 0;

		strOutString = "";

		while (fContinueSearch)
		{
			switch (pxmlRead.NodeType) 
			{
				case XmlNodeType.Element:
					if (pxmlRead.Name == strSearchElement)
					{
						if (iCnt++ < iElementSkipCount) break;

						fElementFound = true;

//						Console.Write("Element<{0}> ", pxmlRead.Name);
						if (null != strSearchAttribute)
						{
							if ((0 != pxmlRead.AttributeCount) && (pxmlRead.MoveToAttribute(strSearchAttribute)))
							{
								fContinueSearch = false;

								while (pxmlRead.ReadAttributeValue())
								{
									strOutString = pxmlRead.Value;
//									ECUtils.LogWrite("Attrib<{0}>= '{1}'", strSearchAttribute, strOutString);
								} 
							}
							else
							{
//								ECUtils.LogWrite("Error: No Attrib<{0}> ", strSearchAttribute);
							}
						}
                        if (pxmlRead.IsEmptyElement)
                            fContinueSearch = false;
                    }
                    break;
				case XmlNodeType.Text:
					if (fElementFound)
					{
						fContinueSearch = false;

						strOutString = pxmlRead.Value;
						//Console.Write("Text: <{0}>\n", strOutString);
					}
					break;
				case XmlNodeType.CDATA:
					//					Console.Write("CDATA: <![CDATA[{0}]]>\n", pxmlRead.Value);
					break;
				case XmlNodeType.ProcessingInstruction:
					//					Console.Write("<?{0} {1}?>\n", pxmlRead.Name, pxmlRead.Value);
					break;
				case XmlNodeType.Comment:
					//					Console.Write("<!--{0}-->", pxmlRead.Value);
					break;
				case XmlNodeType.XmlDeclaration:
					//					Console.Write("Comment: <?xml version='1.0'?>\n");
					break;
				case XmlNodeType.Document:
					//					Console.Write("Document: \n");
					break;
				case XmlNodeType.DocumentType:
					//					Console.Write("DocumentType: <!DOCTYPE {0} [{1}]\n", pxmlRead.Name, pxmlRead.Value);
					break;
				case XmlNodeType.EntityReference:
					//					Console.Write("EntityReference: <{0}>\n", pxmlRead.Name);
					break;
				case XmlNodeType.EndElement:
					//					Console.Write("EndElement: </{0}>\n", pxmlRead.Name);
                    if (pxmlRead.Name == strSearchElement)
                        fContinueSearch = false;
                    break;
				default :
					//					ECUtils.LogWrite("default:  Name = '{0}' Value = '{1}'", pxmlRead.Name, pxmlRead.Value);
					break;
			}       
			if (!pxmlRead.Read())
				break;
		} // end of While Loop

	}

	/////////////////////////////////////////////////////////////////////////

	public static void WriteToXML(StreamReader poData,  int iElementSkipCount,
		string strSearchParent, string strSearchElement, string strSearchAttribute, 
		string strInString)
	{
		// Create the XmlDocument.
		XmlDocument xmlDoc = new XmlDocument();
		xmlDoc.Load(poData);

		WriteToXML(xmlDoc, iElementSkipCount, strSearchParent, strSearchElement, strSearchAttribute, strInString);
	}

	/////////////////////////////////////////////////////////////////////////
	public static void WriteToXML(ref string pstrData,  int iElementSkipCount,
		string strSearchParent, string strSearchElement, string strSearchAttribute, 
		string strInString)
	{
		if (pstrData.Length == 0)
			return;

		// Create the XmlDocument.
		XmlDocument xmlDoc = new XmlDocument();
		xmlDoc.Load(new StringReader(pstrData));

		WriteToXML(xmlDoc, iElementSkipCount, strSearchParent, strSearchElement, strSearchAttribute, strInString);

		TextWriter stringWriter = new StringWriter();
		xmlDoc.Save(stringWriter);
		pstrData = stringWriter.ToString();
//		ECUtils.LogWrite(stringWriter.ToString());

//		xmlDoc.Close();
	}

	/////////////////////////////////////////////////////////////////////////
	// Writes either an Element Text Value or an Element's Attribute.
	// If Attribute search param is set to null then we are looking 
	// for Element Text Value
	//
	public static void WriteToXML(XmlDocument xmlDoc, int iElementSkipCount,
		string strSearchParent, string strSearchElement, string strSearchAttribute, 
		string strInString)
	{
#if xDEBUG
		ECUtils.LogWrite("Set Element_{0} <{1}> to {2}", iElementSkipCount, strSearchElement, strInString);
#endif


		XmlNodeList elementList = xmlDoc.GetElementsByTagName(strSearchElement);

        if (0 == elementList.Count) return;

		// Search for matching parent
        while ("" != strSearchParent  &&  elementList.Item(iElementSkipCount).ParentNode.Name != strSearchParent)
		{
			string str = elementList.Item(iElementSkipCount).ParentNode.Name;
			iElementSkipCount++;
            if (iElementSkipCount >= elementList.Count) return;
        }

		if (null == strSearchAttribute)
		{
			if (null != elementList)
				// write element value
				elementList.Item(iElementSkipCount).InnerText = strInString;
			else
                ECUtils.LogWrite("Has No Element<" + strSearchElement.ToString()+ "> ");
		}
		else	// we are searching for an Attribute
		{
			if ((null != elementList.Item(iElementSkipCount)) && 
			    (null != elementList.Item(iElementSkipCount).Attributes) && 
				(null != elementList.Item(iElementSkipCount).Attributes[strSearchAttribute]) )
			{
				// write Attribute value
				elementList.Item(iElementSkipCount).Attributes[strSearchAttribute].Value = strInString;
			}
			else
				ECUtils.LogWrite("Has No Attrib<" + strSearchAttribute.ToString() + "> ");
		}

	}

	/////////////////////////////////////////////////////////////////////////
	static int readXmlFile2(ref string stream, out string strName)
		//	static int readXmlFile2( )
		//		static void Main(string[] args)
	{
		strName = "JoseOffice (00:50:c2:4f:a0:07)";

		String sContactID, sDuplicateID;
		String sXml =
			@"<BusinessEntity xsi:type='DynamicEntity' Name='contact'
				xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance'
				xmlns='http://schemas.microsoft.com/crm/2006/WebServices'>
					<Properties>
						<Property xsi:type='KeyProperty' Name='contactid'>
							<Value>{80FB716B-B189-DA11-84BD-0003FFC6732A}</Value>
						</Property>
						<Property xsi:type='StringProperty' Name='duplicateid'>
							<Value>{8e0c0022-2078-4017-9630-6453e80695c0}</Value>
						</Property>
					</Properties>
				</BusinessEntity>";

		XmlDocument xmlDoc = new XmlDocument();
		XmlNamespaceManager xmlNSMgr = new XmlNamespaceManager(xmlDoc.NameTable);

		try
		{
			xmlDoc.LoadXml(sXml);

			// define default NS using prefix "def"
			xmlNSMgr.AddNamespace("def", "http://schemas.microsoft.com/crm/2006/WebServices");

			ECUtils.LogWrite("Here 1");
			// get contactid
			sContactID =
				xmlDoc.SelectSingleNode("//def:Property[@Name='contactid']/def:Value", xmlNSMgr).InnerText;
			ECUtils.LogWrite("Here 2");

			// get duplicateid
			sDuplicateID =
				xmlDoc.SelectSingleNode("//def:Property[@Name='duplicateid']/def:Value", xmlNSMgr).InnerText;

			ECUtils.LogWrite("Contact ID: " + sContactID);
			ECUtils.LogWrite("Duplicate ID: " + sDuplicateID);
		}
		catch (XmlException e)
		{
			ECUtils.LogWrite("Error loading XML: " + e.Message);
		}
		catch (Exception e)
		{
			ECUtils.LogWrite("Error: " + e.Message);
		}

		Console.Write("\nPress any key to continue: ");
		//		Console.ReadKey(true);
		return 0;
	}

	/////////////////////////////////////////////////////////////////////////
	/* This enumeration lists all of the character attributes. 
		   You can combine attributes to achieve specific effects.*/
	public enum CharacterAttributes
	{
		FOREGROUND_WHITE = 0x0007,
		FOREGROUND_BLUE = 0x0001,
		FOREGROUND_GREEN = 0x0002,
		FOREGROUND_RED = 0x0004,
		FOREGROUND_INTENSITY = 0x0008,
		BACKGROUND_BLUE = 0x0010,
		BACKGROUND_GREEN = 0x0020,
		BACKGROUND_RED = 0x0040,
		BACKGROUND_INTENSITY = 0x0080,
		COMMON_LVB_LEADING_BYTE = 0x0100,
		COMMON_LVB_TRAILING_BYTE = 0x0200,
		COMMON_LVB_GRID_HORIZONTAL = 0x0400,
		COMMON_LVB_GRID_LVERTICAL = 0x0800,
		COMMON_LVB_GRID_RVERTICAL = 0x1000,
		COMMON_LVB_REVERSE_VIDEO = 0x4000,
		COMMON_LVB_UNDERSCORE = 0x8000
	}

    static StreamWriter m_oLog = null;

    static IntPtr hConsole = (IntPtr)null;

	public static string	strTstJobStream =
        @"<Data type='JobData' rev='1.0'>
<LaserSignalOn />
<LongDelay>3000000</LongDelay>
<LaserSignalOff />
<Set id='JumpSpeed'>15,5</Set>
<Set id='MarkSpeed'>15,5</Set>
<JumpAbs>-15000,-15000</JumpAbs>
<Set id='Wobble'>1000,100</Set>
<WobbleEnable>1</WobbleEnable>
<MarkAbs>15000,-15000</MarkAbs>
<MarkAbs>15000,15000</MarkAbs>
<MarkAbs>-15000,15000</MarkAbs>
<MarkAbs>-15000,-15000</MarkAbs>
<MarkAbs>15000,-15000</MarkAbs>
<MarkAbs>15000,15000</MarkAbs>
<MarkAbs>-15000,15000</MarkAbs>
<MarkAbs>-15000,-15000</MarkAbs>
<MarkAbs>15000,-15000</MarkAbs>
<MarkAbs>15000,15000</MarkAbs>
<MarkAbs>-15000,15000</MarkAbs>
<MarkAbs>-15000,-15000</MarkAbs>
<MarkAbs>15000,-15000</MarkAbs>
<MarkAbs>15000,15000</MarkAbs>
<MarkAbs>-15000,15000</MarkAbs>
<MarkAbs>-15000,-15000</MarkAbs>
<WobbleEnable>0</WobbleEnable>
<LongDelay>1000000</LongDelay>
<Draw>
	<Vertex>15000,-15000</Vertex>
	<Vertex>15000,15000</Vertex>
	<Vertex>-15000,15000</Vertex>
	<Vertex>-15000,-15000</Vertex>
</Draw>
</Data>";

	
	public static string	strTstJobStream1 =
        @"<Data type='JobData' rev='1.0'>
<BeginJob />
<Set id='JumpSpeed'>15,5</Set>
<Set id='MarkSpeed'>15,5</Set>
<Set id='MotfCalFactor'>0.5</Set>
<WaitForIO>2,1,10000000,0</WaitForIO>
<JumpAbs>-15000,-15000</JumpAbs>
<MarkAbs>15000,-15000</MarkAbs>
<MarkAbs>15000,15000</MarkAbs>
<MarkAbs>-15000,15000</MarkAbs>
<MarkAbs>-15000,-15000</MarkAbs>
<MarkAbs>15000,15000</MarkAbs>
<MarkAbs>15000,-15000</MarkAbs>
<MarkAbs>-15000,15000</MarkAbs>
<MarkAbs>-15000,-15000</MarkAbs>
<MarkAbs>15000,-15000</MarkAbs>
<MarkAbs>15000,15000</MarkAbs>
<MarkAbs>-15000,15000</MarkAbs>
<MarkAbs>-15000,-15000</MarkAbs>
<MarkAbs>15000,15000</MarkAbs>
<MarkAbs>15000,-15000</MarkAbs>
<MarkAbs>-15000,15000</MarkAbs>
<MarkAbs>-15000,-15000</MarkAbs>
<MarkAbs>15000,-15000</MarkAbs>
<MarkAbs>15000,15000</MarkAbs>
<MarkAbs>-15000,15000</MarkAbs>
<MarkAbs>-15000,-15000</MarkAbs>
<MarkAbs>15000,15000</MarkAbs>
<MarkAbs>15000,-15000</MarkAbs>
<MarkAbs>-15000,15000</MarkAbs>
<MarkAbs>-15000,-15000</MarkAbs>
<LongDelay>1000000</LongDelay>
<MarkAbs>500,-500</MarkAbs>
<MarkAbs>500,500</MarkAbs>
<MarkAbs>-500,500</MarkAbs>
<MarkAbs>-500,-500</MarkAbs>
<MarkAbs>500,500</MarkAbs>
<MarkAbs>500,-500</MarkAbs>
<MarkAbs>-500,500</MarkAbs>
<MarkAbs>-500,-500</MarkAbs>
<MarkAbs>500,-500</MarkAbs>
<MarkAbs>500,500</MarkAbs>
<MarkAbs>-500,500</MarkAbs>
<MarkAbs>-500,-500</MarkAbs>
<MarkAbs>500,500</MarkAbs>
<MarkAbs>500,-500</MarkAbs>
<MarkAbs>-500,500</MarkAbs>
<MarkAbs>-500,-500</MarkAbs>
<MarkAbs>500,-500</MarkAbs>
<MarkAbs>500,500</MarkAbs>
<MarkAbs>-500,500</MarkAbs>
<MarkAbs>-500,-500</MarkAbs>
<MarkAbs>500,500</MarkAbs>
<MarkAbs>500,-500</MarkAbs>
<MarkAbs>-500,500</MarkAbs>
<MarkAbs>-500,-500</MarkAbs>
<JumpAbs>-15000,-15000</JumpAbs>
<EndJob />
</Data>";

	//	<ApplicationEvent>65,1</ApplicationEvent>
	//	<ApplicationEvent>66,1</ApplicationEvent>

}

